using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Globalization;
using System.Reflection;
using System.Text;
using System.Transactions;
using Produmex.Foundation;
using Produmex.Foundation.Barcode;
using Produmex.Foundation.Data;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Convertors;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.GS1;
using Produmex.Foundation.Messages;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.DataObjects;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Foundation.SlimScreen.WinGui;
using Produmex.Foundation.Utilities;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo;
using Produmex.Foundation.Wwf.Sbo.DataObjects;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Devices;
using Produmex.Sbo.Logex.Data.Generators.BatchNumberGenerators;
using Produmex.Sbo.Logex.Data.Generators.PackingController;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.Data.ViewObjects.Definitions.Views;

// PmxWorkflowExecutionTypes.HOOK_FLOW
namespace Produmex.Sbo.Logex.WorkflowScripts
{
    public class WorkflowScript_AfterItemPickedHookScript : WorkflowInstanceScriptBase
    {
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        // Input parameters * do not change *
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;

        public ReadOnlyBinder<Collection<PickingItemInfo>> PickedItemInfos;
        public ReadOnlyBinder<int?> LUID;
        public ReadOnlyBinder<Collection<string>> ListOfSerialNumbers;
        public ReadOnlyBinder<int> DocEntry;
        public ReadOnlyBinder<string> ObjType;
        
        // Sub-flows
        // <none>


        public WorkflowScript_AfterItemPickedHookScript(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
            : base(parent, factory)
        {
        }

        #region WorkflowInstanceScriptBase Members

        protected override void Execute()
        {
            // Parameters in scope
            Session session = GetScopeParameter("Session") as Session;
            ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;
            Message msg = null;


            string query = null;
            string initialErrorKey = null;
            DataSet dsContainer = null;
            string containerNumber = null;
            string enteredContainerNumber = null;
            const string UdfContainerNumber = "U_PMX_WEB_CNT_NR";
            //bool goBack = false;

        // query to get container number for the linked sales order line
        // screen to show container number
        Step_ShowContainerNumber:

            #region SHOW CONTAINER NUMBER
            query = BuildQuery.GetContainerNumber(LUID, UdfContainerNumber);
            dsContainer = sboProviderService.RunView(false, null, null, query);

            // add check to see if the query returns a result - if not, its not for sales and we don't need any action
            if (dsContainer.Tables[0].Rows.Count == 0)
            {
                return;
            }
            else
            {
                containerNumber = Convert.ToString(dsContainer.Tables[0].Rows[0][UdfContainerNumber]);

                // do not do anything if the line does not have a container number
                if (String.IsNullOrWhiteSpace(containerNumber) || string.IsNullOrEmpty(containerNumber))
                {
                    return;
                }
                else
                {


                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "MessageKey", "Please go to container : " + containerNumber,
                            "NoTranslationOfMessageKey", true,
                            "ShowButton", true));
                    WaitForMessage();
                //goBack = false;


                #endregion


                Step_ScanContainerNumber:

                    #region SCAN CONTAINER NUMBER

                    // input screen to scan container number

                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterStringValueScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                                 "InitialErrorKey", initialErrorKey,
                                 "TitleKey", "Please scan the container",
                                 "ForceDataEntry", true,
                                 "ForceUppercase", true,
                                 "AllowToGoBack", true
                                 ));
                    msg = WaitForMessage();
                    //goBack = false;

                    // reset error
                    initialErrorKey = null;

                    if (msg.Name.EndsWith(".StringEntered"))
                    {
                        enteredContainerNumber = ExtractParameter<string>(msg.Parameters, "stringValue");
                        // check if it matches
                        if (enteredContainerNumber != containerNumber)
                        {
                            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                                DefaultCultureInfo.Get(), BuildParamCollection(
                                    "MessageKey", "The scanned container " + enteredContainerNumber + " is not correct. The correct container is " + containerNumber,
                                    "IsError", true,
                                    "ShowButton", true));
                            WaitForMessage();

                            goto Step_ScanContainerNumber;

                        }
                        else
                        {
                            return;
                        }
                    }
                    if (msg.Name.EndsWith(".BackRequested"))
                    {
                        //goBack = true;
                        goto Step_ShowContainerNumber;
                    }

                    #endregion

                }
            }

        }

        #endregion



        #region BUILD QUERY


        /// <summary>
        /// Static class grouping our native queries
        /// </summary>
        internal static class BuildQuery
        {

            /// <summary>
            /// Gets the container number.
            /// </summary>
            /// <returns></returns>
            public static string GetContainerNumber(int? luid, string UdfDefinition)
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT ");

                SqlCommandHelper.AddTableColumnName(strBuilder, PmxSalesOrderLineTable.NAME, UdfDefinition);

                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxMoveLineTable.NAME));

                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxPickListLineTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.DocEntry.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.BaseEntry.NAME);
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.LineNum.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.BaseLine.NAME);
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.BaseType.NAME);
                strBuilder.Append(" = 'PMX_PLHE' ");

                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxPickListProposalLineTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.DocEntry.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.BaseEntry.NAME);
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.LineNum.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.BaseLine.NAME);
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.BaseType.NAME);
                strBuilder.Append(" = 'PMX_PLPH' ");

                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxSalesOrderLineTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxSalesOrderLineTable.NAME, PmxSalesOrderLineTable.Columns.DocEntry.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.BaseEntry.NAME);
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxSalesOrderLineTable.NAME, PmxSalesOrderLineTable.Columns.LineNum.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.BaseLine.NAME);
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.BaseType.NAME);
                strBuilder.Append(" = '17' ");

                strBuilder.Append(" WHERE ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.DestinationLogisticUnitIdentificationKey.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(luid, CultureInfo.InvariantCulture));

                return strBuilder.ToString();
            }

        }

        #endregion


       
    }


}